# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/state_update.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.args import fco_args_pb2 as tecton__proto_dot_args_dot_fco__args__pb2
from tecton_proto.args import repo_metadata_pb2 as tecton__proto_dot_args_dot_repo__metadata__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/data/state_update.proto',
  package='tecton_proto.data',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.dataP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n$tecton_proto/data/state_update.proto\x12\x11tecton_proto.data\x1a\x1ctecton_proto/common/id.proto\x1a tecton_proto/args/fco_args.proto\x1a%tecton_proto/args/repo_metadata.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"|\n\x0f\x46\x63oPropertyDiff\x12#\n\rproperty_name\x18\x01 \x01(\tR\x0cpropertyName\x12!\n\x0cval_existing\x18\x02 \x01(\tR\x0bvalExisting\x12!\n\x0cval_declared\x18\x03 \x01(\tR\x0bvalDeclared\"\x83\x02\n\x07\x46\x63oDiff\x12\x38\n\x04type\x18\x01 \x01(\x0e\x32$.tecton_proto.data.FcoTransitionTypeR\x04type\x12?\n\rexisting_args\x18\x07 \x01(\x0b\x32\x1a.tecton_proto.args.FcoArgsR\x0c\x65xistingArgs\x12?\n\rdeclared_args\x18\x08 \x01(\x0b\x32\x1a.tecton_proto.args.FcoArgsR\x0c\x64\x65\x63laredArgs\x12\x36\n\x04\x64iff\x18\x04 \x03(\x0b\x32\".tecton_proto.data.FcoPropertyDiffR\x04\x64iffJ\x04\x08\t\x10\n\"C\n\x0b\x46\x63oFieldRef\x12.\n\x06\x66\x63o_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x05\x66\x63oIdJ\x04\x08\x02\x10\x03\"h\n\x11ValidationMessage\x12\x18\n\x07message\x18\x01 \x01(\tR\x07message\x12\x39\n\x08\x66\x63o_refs\x18\x02 \x03(\x0b\x32\x1e.tecton_proto.data.FcoFieldRefR\x07\x66\x63oRefs\"\x92\x01\n\x10ValidationResult\x12<\n\x06\x65rrors\x18\x01 \x03(\x0b\x32$.tecton_proto.data.ValidationMessageR\x06\x65rrors\x12@\n\x08warnings\x18\x02 \x03(\x0b\x32$.tecton_proto.data.ValidationMessageR\x08warnings\"\xb4\x02\n\x12StateUpdateRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12\x35\n\x08\x66\x63o_args\x18\x02 \x03(\x0b\x32\x1a.tecton_proto.args.FcoArgsR\x07\x66\x63oArgs\x12R\n\x10repo_source_info\x18\x03 \x01(\x0b\x32(.tecton_proto.args.FeatureRepoSourceInfoR\x0erepoSourceInfo\x12\x1f\n\x0bupgrade_all\x18\x04 \x01(\x08R\nupgradeAll\x12!\n\x0crequested_by\x18\x05 \x01(\tR\x0brequestedBy\x12\x1f\n\x0bsdk_version\x18\t \x01(\tR\nsdkVersionJ\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08J\x04\x08\x08\x10\t\"\xa7\x01\n\x10StateUpdateEntry\x12\x1b\n\tcommit_id\x18\x01 \x01(\tR\x08\x63ommitId\x12\x1d\n\napplied_by\x18\x02 \x01(\tR\tappliedBy\x12\x39\n\napplied_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tappliedAt\x12\x1c\n\tworkspace\x18\x04 \x01(\tR\tworkspace*n\n\x11\x46\x63oTransitionType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06\x43REATE\x10\x01\x12\n\n\x06\x44\x45LETE\x10\x02\x12\x0b\n\x07UPGRADE\x10\x05\x12\n\n\x06UPDATE\x10\x03\x12\x0c\n\x08RECREATE\x10\x04\x12\r\n\tUNCHANGED\x10\x06\x42\x13\n\x0f\x63om.tecton.dataP\x01'
  ,
  dependencies=[tecton__proto_dot_common_dot_id__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_fco__args__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_repo__metadata__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])

_FCOTRANSITIONTYPE = _descriptor.EnumDescriptor(
  name='FcoTransitionType',
  full_name='tecton_proto.data.FcoTransitionType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CREATE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DELETE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UPGRADE', index=3, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UPDATE', index=4, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RECREATE', index=5, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNCHANGED', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1388,
  serialized_end=1498,
)
_sym_db.RegisterEnumDescriptor(_FCOTRANSITIONTYPE)

FcoTransitionType = enum_type_wrapper.EnumTypeWrapper(_FCOTRANSITIONTYPE)
UNKNOWN = 0
CREATE = 1
DELETE = 2
UPGRADE = 5
UPDATE = 3
RECREATE = 4
UNCHANGED = 6



_FCOPROPERTYDIFF = _descriptor.Descriptor(
  name='FcoPropertyDiff',
  full_name='tecton_proto.data.FcoPropertyDiff',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='property_name', full_name='tecton_proto.data.FcoPropertyDiff.property_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='propertyName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='val_existing', full_name='tecton_proto.data.FcoPropertyDiff.val_existing', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='valExisting', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='val_declared', full_name='tecton_proto.data.FcoPropertyDiff.val_declared', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='valDeclared', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=195,
  serialized_end=319,
)


_FCODIFF = _descriptor.Descriptor(
  name='FcoDiff',
  full_name='tecton_proto.data.FcoDiff',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='tecton_proto.data.FcoDiff.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='type', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='existing_args', full_name='tecton_proto.data.FcoDiff.existing_args', index=1,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='existingArgs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='declared_args', full_name='tecton_proto.data.FcoDiff.declared_args', index=2,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='declaredArgs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='diff', full_name='tecton_proto.data.FcoDiff.diff', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='diff', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=322,
  serialized_end=581,
)


_FCOFIELDREF = _descriptor.Descriptor(
  name='FcoFieldRef',
  full_name='tecton_proto.data.FcoFieldRef',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fco_id', full_name='tecton_proto.data.FcoFieldRef.fco_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='fcoId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=583,
  serialized_end=650,
)


_VALIDATIONMESSAGE = _descriptor.Descriptor(
  name='ValidationMessage',
  full_name='tecton_proto.data.ValidationMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='tecton_proto.data.ValidationMessage.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='message', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fco_refs', full_name='tecton_proto.data.ValidationMessage.fco_refs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='fcoRefs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=652,
  serialized_end=756,
)


_VALIDATIONRESULT = _descriptor.Descriptor(
  name='ValidationResult',
  full_name='tecton_proto.data.ValidationResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='errors', full_name='tecton_proto.data.ValidationResult.errors', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='errors', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='warnings', full_name='tecton_proto.data.ValidationResult.warnings', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='warnings', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=759,
  serialized_end=905,
)


_STATEUPDATEREQUEST = _descriptor.Descriptor(
  name='StateUpdateRequest',
  full_name='tecton_proto.data.StateUpdateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='workspace', full_name='tecton_proto.data.StateUpdateRequest.workspace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workspace', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fco_args', full_name='tecton_proto.data.StateUpdateRequest.fco_args', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='fcoArgs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='repo_source_info', full_name='tecton_proto.data.StateUpdateRequest.repo_source_info', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='repoSourceInfo', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='upgrade_all', full_name='tecton_proto.data.StateUpdateRequest.upgrade_all', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='upgradeAll', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='requested_by', full_name='tecton_proto.data.StateUpdateRequest.requested_by', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='requestedBy', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sdk_version', full_name='tecton_proto.data.StateUpdateRequest.sdk_version', index=5,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sdkVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=908,
  serialized_end=1216,
)


_STATEUPDATEENTRY = _descriptor.Descriptor(
  name='StateUpdateEntry',
  full_name='tecton_proto.data.StateUpdateEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='commit_id', full_name='tecton_proto.data.StateUpdateEntry.commit_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='commitId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='applied_by', full_name='tecton_proto.data.StateUpdateEntry.applied_by', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='appliedBy', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='applied_at', full_name='tecton_proto.data.StateUpdateEntry.applied_at', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='appliedAt', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workspace', full_name='tecton_proto.data.StateUpdateEntry.workspace', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workspace', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1219,
  serialized_end=1386,
)

_FCODIFF.fields_by_name['type'].enum_type = _FCOTRANSITIONTYPE
_FCODIFF.fields_by_name['existing_args'].message_type = tecton__proto_dot_args_dot_fco__args__pb2._FCOARGS
_FCODIFF.fields_by_name['declared_args'].message_type = tecton__proto_dot_args_dot_fco__args__pb2._FCOARGS
_FCODIFF.fields_by_name['diff'].message_type = _FCOPROPERTYDIFF
_FCOFIELDREF.fields_by_name['fco_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_VALIDATIONMESSAGE.fields_by_name['fco_refs'].message_type = _FCOFIELDREF
_VALIDATIONRESULT.fields_by_name['errors'].message_type = _VALIDATIONMESSAGE
_VALIDATIONRESULT.fields_by_name['warnings'].message_type = _VALIDATIONMESSAGE
_STATEUPDATEREQUEST.fields_by_name['fco_args'].message_type = tecton__proto_dot_args_dot_fco__args__pb2._FCOARGS
_STATEUPDATEREQUEST.fields_by_name['repo_source_info'].message_type = tecton__proto_dot_args_dot_repo__metadata__pb2._FEATUREREPOSOURCEINFO
_STATEUPDATEENTRY.fields_by_name['applied_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name['FcoPropertyDiff'] = _FCOPROPERTYDIFF
DESCRIPTOR.message_types_by_name['FcoDiff'] = _FCODIFF
DESCRIPTOR.message_types_by_name['FcoFieldRef'] = _FCOFIELDREF
DESCRIPTOR.message_types_by_name['ValidationMessage'] = _VALIDATIONMESSAGE
DESCRIPTOR.message_types_by_name['ValidationResult'] = _VALIDATIONRESULT
DESCRIPTOR.message_types_by_name['StateUpdateRequest'] = _STATEUPDATEREQUEST
DESCRIPTOR.message_types_by_name['StateUpdateEntry'] = _STATEUPDATEENTRY
DESCRIPTOR.enum_types_by_name['FcoTransitionType'] = _FCOTRANSITIONTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FcoPropertyDiff = _reflection.GeneratedProtocolMessageType('FcoPropertyDiff', (_message.Message,), {
  'DESCRIPTOR' : _FCOPROPERTYDIFF,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.FcoPropertyDiff)
  })
_sym_db.RegisterMessage(FcoPropertyDiff)

FcoDiff = _reflection.GeneratedProtocolMessageType('FcoDiff', (_message.Message,), {
  'DESCRIPTOR' : _FCODIFF,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.FcoDiff)
  })
_sym_db.RegisterMessage(FcoDiff)

FcoFieldRef = _reflection.GeneratedProtocolMessageType('FcoFieldRef', (_message.Message,), {
  'DESCRIPTOR' : _FCOFIELDREF,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.FcoFieldRef)
  })
_sym_db.RegisterMessage(FcoFieldRef)

ValidationMessage = _reflection.GeneratedProtocolMessageType('ValidationMessage', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATIONMESSAGE,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.ValidationMessage)
  })
_sym_db.RegisterMessage(ValidationMessage)

ValidationResult = _reflection.GeneratedProtocolMessageType('ValidationResult', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATIONRESULT,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.ValidationResult)
  })
_sym_db.RegisterMessage(ValidationResult)

StateUpdateRequest = _reflection.GeneratedProtocolMessageType('StateUpdateRequest', (_message.Message,), {
  'DESCRIPTOR' : _STATEUPDATEREQUEST,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.StateUpdateRequest)
  })
_sym_db.RegisterMessage(StateUpdateRequest)

StateUpdateEntry = _reflection.GeneratedProtocolMessageType('StateUpdateEntry', (_message.Message,), {
  'DESCRIPTOR' : _STATEUPDATEENTRY,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.StateUpdateEntry)
  })
_sym_db.RegisterMessage(StateUpdateEntry)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/freshness_status.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/data/freshness_status.proto',
  package='tecton_proto.data',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.dataP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n(tecton_proto/data/freshness_status.proto\x12\x11tecton_proto.data\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1ctecton_proto/common/id.proto\"\xad\x03\n\x0f\x46reshnessStatus\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12*\n\x11\x66\x65\x61ture_view_name\x18\x02 \x01(\tR\x0f\x66\x65\x61tureViewName\x12\x39\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12H\n\x12\x65xpected_freshness\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationR\x11\x65xpectedFreshness\x12\x1b\n\tis_stream\x18\x05 \x01(\x08R\x08isStream\x12\x37\n\x17materialization_enabled\x18\x06 \x01(\x08R\x16materializationEnabled\x12\x37\n\tfreshness\x18\x07 \x01(\x0b\x32\x19.google.protobuf.DurationR\tfreshness\x12\x19\n\x08is_stale\x18\x08 \x01(\x08R\x07isStaleB\x13\n\x0f\x63om.tecton.dataP\x01'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_id__pb2.DESCRIPTOR,])




_FRESHNESSSTATUS = _descriptor.Descriptor(
  name='FreshnessStatus',
  full_name='tecton_proto.data.FreshnessStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_view_id', full_name='tecton_proto.data.FreshnessStatus.feature_view_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view_name', full_name='tecton_proto.data.FreshnessStatus.feature_view_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='tecton_proto.data.FreshnessStatus.created_at', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='createdAt', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expected_freshness', full_name='tecton_proto.data.FreshnessStatus.expected_freshness', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='expectedFreshness', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_stream', full_name='tecton_proto.data.FreshnessStatus.is_stream', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='isStream', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='materialization_enabled', full_name='tecton_proto.data.FreshnessStatus.materialization_enabled', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='materializationEnabled', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='freshness', full_name='tecton_proto.data.FreshnessStatus.freshness', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='freshness', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_stale', full_name='tecton_proto.data.FreshnessStatus.is_stale', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='isStale', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=159,
  serialized_end=588,
)

_FRESHNESSSTATUS.fields_by_name['feature_view_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_FRESHNESSSTATUS.fields_by_name['created_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_FRESHNESSSTATUS.fields_by_name['expected_freshness'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_FRESHNESSSTATUS.fields_by_name['freshness'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
DESCRIPTOR.message_types_by_name['FreshnessStatus'] = _FRESHNESSSTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FreshnessStatus = _reflection.GeneratedProtocolMessageType('FreshnessStatus', (_message.Message,), {
  'DESCRIPTOR' : _FRESHNESSSTATUS,
  '__module__' : 'tecton_proto.data.freshness_status_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.FreshnessStatus)
  })
_sym_db.RegisterMessage(FreshnessStatus)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import datetime

import pkg_resources


def get_version():
    try:
        return pkg_resources.get_distribution("tecton").version
    except pkg_resources.DistributionNotFound:
        return None


def get_status():
    try:
        from tecton._stamp import BUILD_STATUS

        return BUILD_STATUS
    except ImportError:
        return {}


def get_hash() -> str:
    status = get_status()
    return status.get("GIT_COMMIT", "n/a")


def summary():
    status = get_status()

    ts_seconds = status.get("BUILD_TIMESTAMP", None)
    ts = datetime.datetime.utcfromtimestamp(int(ts_seconds)).isoformat() if ts_seconds else "n/a"

    commit = status.get("GIT_COMMIT", "n/a")

    print(f"Version: {get_version() or 'n/a'}")
    print(f"Git Commit: {commit}")
    print(f"Build Datetime: {ts}")



from typing import List

from google.protobuf import duration_pb2

from tecton_proto.common.schema_pb2 import Schema as SchemaProto
from tecton_proto.data.feature_store_pb2 import FeatureStoreFormatVersion

CONTINUOUS_MODE_BATCH_INTERVAL = duration_pb2.Duration(seconds=86400)


def get_input_feature_columns(view_schema: SchemaProto, join_keys: List[str], timestamp_key: str) -> List[str]:
    column_names = (c.name for c in view_schema.columns)
    return [c for c in column_names if c not in join_keys and c != timestamp_key]


def validate_version(version):
    assert (
        version >= FeatureStoreFormatVersion.FEATURE_STORE_FORMAT_VERSION_DEFAULT
        or version <= FeatureStoreFormatVersion.FEATURE_STORE_FORMAT_VERSION_MAX
    )



from tecton._internals.fco import Fco
from tecton_proto.args.feature_view_pb2 import FeatureViewArgs
from tecton_proto.args.repo_metadata_pb2 import SourceInfo
from tecton_proto.common.id_pb2 import Id


class FeatureDefinition(Fco):
    """
    Represents the base class for Declarative FeatureViews and FeatureTables
    """

    _args: FeatureViewArgs
    _source_info: SourceInfo

    def _id(self) -> Id:
        return self._args.feature_view_id

    @property
    def name(self) -> str:
        """
        Name of this Tecton Object.
        """
        return self._args.info.name

    def __getitem__(self, features):
        from tecton.feature_services.feature_service_args import FeaturesConfig

        return FeaturesConfig(feature_view=self, namespace=self.name, features=features)



from typing import Dict
from typing import List
from typing import Optional
from typing import Union

import attr
import pandas
import pendulum
from pyspark.sql import DataFrame
from pyspark.sql import functions
from pyspark.sql import SparkSession
from pyspark.sql.functions import isnull
from pyspark.sql.functions import to_timestamp
from pyspark.sql.functions import when
from pyspark.sql.utils import AnalysisException

import tecton
import tecton_spark
from tecton._internals import errors
from tecton._internals.utils import is_bfc_mode_single
from tecton.tecton_context import TectonContext
from tecton_spark.feature_definition_wrapper import FeatureDefinitionWrapper as FeatureDefinition
from tecton_spark.feature_set_config import FeatureSetConfig
from tecton_spark.logger import get_logger
from tecton_spark.materialization_params import MaterializationParams
from tecton_spark.schema_spark_utils import schema_to_spark
from tecton_spark.spark_time_utils import add_seconds_to_timestamp
from tecton_spark.spark_time_utils import convert_epoch_to_datetime

TEMPORAL_JOIN_COLUMN = "__tecton_temporal_join_column"
TEMPORARY_COLUMN_NAME = "__temp_time_key"

logger = get_logger("Internal")


@attr.s(auto_attribs=True)
class _FeatureSetQueryBuilder(object):
    spark: SparkSession
    spine_df: Optional[DataFrame]
    # Dataframe against which we're joining this FeatureView.
    # It can be None *only* when we're doing an outer join or
    # for FeatureSetConfigs containing OnDemandFeatureViews.
    parent_df: Optional[DataFrame]
    fd: FeatureDefinition
    override_join_keys: Dict[str, str]
    namespace: str
    features: List[str]
    spine_time_key: Optional[str]
    use_materialized_data: bool
    include_feature_view_timestamp_columns: bool
    join_type: str
    wildcard_key_not_in_spine: bool = False
    all_spine_join_keys: List[str] = []
    spine_time_limits: Optional[pendulum.Period] = None

    def build(self) -> DataFrame:
        """Build the dataframe by joining the given feature definition to the parent DF using the provided configs."""
        self._validate()
        # If the outer join is specified, we should extract whole data from the feature definition by providing None spine
        fv_spine_df = None
        if self.fd.is_on_demand:
            fv_spine_df = self._construct_spine_for_on_demand_feature_view()
            time_key = self.spine_time_key
        elif self.join_type != "outer":
            fv_spine_df = self._construct_spine_for_materialized_feature_view()
            # this is None because we're using a modified fv_spine_df that matches the FV time key
            time_key = None
        else:
            # this is None because we're using no spine
            time_key = None
        logger.info(f"{self.fd.name}: Building Tile DataFrame [2/3]")
        feature_df = _get_feature_dataframe_with_limits(
            fd=self.fd,
            spine=fv_spine_df,
            spine_time_key=time_key,
            spine_time_limits=self.spine_time_limits,
            wildcard_key_not_in_spine=self.wildcard_key_not_in_spine,
            use_materialized_data=self.use_materialized_data,
            namespace=self.namespace,
        ).to_spark()

        if self.fd.is_on_demand:
            feature_df = self._drop_excluded_feature_columns(feature_df)
            # This DF will contain the features computed by this iteration as well as any features
            # from previous iterations passed in via the spine.
            # columns are already renamed in pipeline_helper.dataframe_with_input
            return feature_df

        parent_df, feature_df = self._add_temporal_join_columns(self.parent_df, feature_df)
        feature_df = self._rename_fv_cols_to_fs_names(feature_df)
        feature_df = self._drop_excluded_feature_columns(feature_df)
        logger.info(f"{self.fd.name}: Generating Join [3/3]")
        join_df = self._join(parent_df, feature_df)
        return join_df

    def _validate(self):
        """Validates that _FeatureSetQueryBuilder was constructed with a good state before proceeding."""
        # If this feature definition is temporal, then we should also have a spine time key. Throw an error otherwise.
        if self.fd.is_on_demand:
            return
        if not self.spine_time_key:
            raise errors.FV_TIME_KEY_MISSING(self.fd.name)
        if self.spine_df and self.spine_time_key not in self.spine_df.columns:
            raise errors.MISSING_SPINE_COLUMN("timestamp_key", self.spine_time_key, self.spine_df.columns)

        # Assert correct join_type and parent_df
        if self.join_type not in ("left", "outer"):
            raise errors.INTERNAL_ERROR(f"_FeatureSetQueryBuilder called with join_type={self.join_type}")
        if self.join_type == "left" and self.parent_df is None:
            raise errors.INTERNAL_ERROR("_FeatureSetQueryBuilder called with empty spine on left join")

        self.use_materialized_data = _validate_and_return_use_materialized_data(self.fd, self.use_materialized_data)

    def _construct_spine_for_materialized_feature_view(self):
        """Return the spine for the provided BFV/SFV."""
        spine_columns = [c.name for c in self.spine_df.schema]
        override_join_keys = {k: v for (k, v) in self.override_join_keys.items() if k in spine_columns}
        # Rename columns to match the FeatureView using the join config and temporal columns names.
        column_projection = {self.spine_time_key: self.fd.timestamp_key, **override_join_keys}
        df = self.spine_df.select(*column_projection.keys()).distinct()
        for k, v in column_projection.items():
            df = df.withColumnRenamed(k, v)
        return df

    def _construct_spine_for_on_demand_feature_view(self):
        """Return the spine for the provided OnDemandFeatureView.
        If the ODFV has any dependent FeatureViews, they would be computed first into the parent_df.
        """
        return self.parent_df

    def _add_temporal_join_columns(self, spine_df, feature_df):
        """Add temporal join columns to both spine_df & feature_df and return them."""
        # Process the spine
        if spine_df is not None:
            if self.fd.is_temporal_aggregate:
                materialization_params = MaterializationParams.from_feature_definition(self.fd)
                if self.join_type == "left":
                    # Returns a column of the most recently scheduled anchor for the given FV before the time given by time_key
                    # TODO: Account for processing delay.
                    feature_endtime = materialization_params.most_recent_anchor(to_timestamp(self.spine_time_key))
                else:
                    # During the outer join temporal column values represent the end range, so we only need to round down spine_time_key to the closest anchor time
                    feature_endtime = convert_epoch_to_datetime(
                        materialization_params.align_timestamp_left(to_timestamp(self.spine_time_key))
                    )

                # Create a temporary column for the temporal join condition on both sides that we will drop later.
                spine_df = spine_df.withColumn(TEMPORAL_JOIN_COLUMN, feature_endtime)
            else:
                # For temporal features just use the spine's original timestamp, since feature_df has the same timestamps
                spine_time_val = to_timestamp(self.spine_time_key)
                spine_df = spine_df.withColumn(TEMPORAL_JOIN_COLUMN, spine_time_val)
        if feature_df is None:
            return spine_df, None

        # Process FeatureView DataFrame
        timestamp_val = to_timestamp(self.fd.timestamp_key)
        feature_df = feature_df.withColumn(TEMPORAL_JOIN_COLUMN, timestamp_val)
        if self.include_feature_view_timestamp_columns:
            if self.fd.is_temporal_aggregate:
                materialization_params = MaterializationParams.from_feature_definition(self.fd)
                # When displaying to users, represent tile with its end time rather than the tile anchor time.
                slide_interval = materialization_params.get_tile_interval().in_seconds()
                adjusted_timestamp = add_seconds_to_timestamp(to_timestamp(self.fd.timestamp_key), slide_interval)
                feature_df = feature_df.withColumn(self.fd.timestamp_key, adjusted_timestamp.cast("timestamp"))

            feature_df = feature_df.withColumnRenamed(self.fd.timestamp_key, f"{self.fd.name}.{self.fd.timestamp_key}")
        else:
            feature_df = feature_df.drop(self.fd.timestamp_key)

        # Drop duplicates from FV Dataframe - they could be introduced by identical anchor times after dropping the time keys
        return spine_df, feature_df.drop_duplicates()

    def _drop_excluded_feature_columns(self, feature_df):
        """Drops feature columns that should not be included.

        This occurs when a feature service includes only a subset of features from a feature view.
        """
        columns_to_drop = [
            f"{self.namespace}.{feature}" for feature in self.fd.features if feature not in self.features
        ]
        if columns_to_drop:
            feature_df = feature_df.drop(*columns_to_drop)
        return feature_df

    def _rename_fv_cols_to_fs_names(self, feature_df):
        """Rename FV columns to FS key space & by prefixing with namespaces if requested."""
        # Rename features if we have a namespace.
        if self.namespace:
            for feature in self.fd.features:
                feature_df = feature_df.withColumnRenamed(feature, f"{self.namespace}.{feature}")
        # Rename FV column names to spine column names to match spine_df
        for spine_column_name, fv_column_name in self.override_join_keys.items():
            feature_df = feature_df.withColumnRenamed(fv_column_name, spine_column_name)
        return feature_df

    def _join(self, parent_df, feature_df) -> DataFrame:
        """Properly join spine_df & feature_df, handling cases such as temporal aggregate vs temporal."""
        # If FV does not have wildcard join_key or if spine contains all the join_keys, simply join
        # feature_df to parent_df on shared set of FV join_keys and timestamp column.
        if not self.wildcard_key_not_in_spine or self.fd.wildcard_join_key is None:
            # Joining columns include all intersecting join keys + time key for temporal features
            # Note, that non-intersecting join keys get cartesian-joined automatically, which is very expensive.
            join_cols = [TEMPORAL_JOIN_COLUMN]
            if parent_df is not None:
                left_join_cols = set(parent_df.columns)
                right_join_cols = self.override_join_keys.keys()
                if left_join_cols - right_join_cols and right_join_cols - left_join_cols:
                    # Cartesian join can only happen during preview - otherwise spine should contain all necessary columns
                    assert self.join_type == "outer"
                    logger.warn(
                        "This FeatureService hits an edge case for which Tecton's preview() method is not optimized, which "
                        "may make this method run slower than usual. To make things faster, try running preview() on individual feature definitions."
                    )
                join_cols += list(left_join_cols & right_join_cols)

            # Do the join if parent_df exists
            df = feature_df if parent_df is None else parent_df.join(feature_df, on=join_cols, how=self.join_type)
        else:
            df = self._join_wildcard_FV_to_wildcard_spine(parent_df, feature_df)

        if self.join_type == "outer":
            df = self._join_for_preview(df)

        return df.drop(TEMPORAL_JOIN_COLUMN)

    def _join_wildcard_FV_to_wildcard_spine(self, parent_df, feature_df) -> DataFrame:
        # Cache `parent_df` as it is used used multiple times below
        parent_df.cache()

        spine_wildcard_key = [
            spine_key for spine_key, fv_key in self.override_join_keys.items() if fv_key == self.fd.wildcard_join_key
        ][0]
        fv_bound_join_keys = list(self.override_join_keys.keys())
        fv_bound_join_keys.remove(spine_wildcard_key)

        # Timestamp key does not necessarily have to be in `spine_join_keys` that will later be used for outer joining.
        # But it is for conveninece to avoid having duplicate timestamp columns after joining.
        spine_join_keys = self.all_spine_join_keys + [TEMPORAL_JOIN_COLUMN]
        if self.spine_time_key:
            spine_join_keys.append(self.spine_time_key)
        s, _ = self._add_temporal_join_columns(self.spine_df, None)
        parent_df_without_features = s.select(spine_join_keys).drop_duplicates()

        # Add all non-self.spine_time_keyildcard join_keys from the other feature definitions in the FeatureService
        # to the `feature_df` so that we can later outer join it with `parent_df`.
        df = feature_df.join(parent_df_without_features, fv_bound_join_keys + [TEMPORAL_JOIN_COLUMN], "inner")

        spine_columns = [column.name for column in parent_df.schema]
        # If parent spine already contains wildcard keys, include it in the outer joining columns.
        if spine_wildcard_key in spine_columns:
            spine_join_keys.append(spine_wildcard_key)

        # Set of fully bound join_keys from the spine can result into different sets of wildcard join_key
        # ranges from different feature definitions.
        # We do outer join to aggregate all the seen values of wildcard join_key column.
        return df.join(parent_df, spine_join_keys, "outer")

    def _join_for_preview(self, feature_df):
        # The outer join means we don't provide spine_df. Therefore we need to somehow generate the time key.
        # Here, we use the time key from the temporal join column. Therefore it's important that the first
        # feature definition in the FeatureSet is temporal aggregate with smallest tile interval, so that the resulting
        # time key has the highest granularity. See _sort_key method for how the feature definitions are sorted.
        if self.spine_time_key not in feature_df.columns:
            df = feature_df.withColumn(self.spine_time_key, functions.col(TEMPORAL_JOIN_COLUMN))
        else:
            # Here we need to basically do an outer join of the original time key and the temporal join column
            # We achieve this by creating a new column with CASE expression (=spine_time_key if not null, otherwise temporal join column)
            df = (
                feature_df.withColumn(
                    TEMPORARY_COLUMN_NAME,
                    when(isnull(self.spine_time_key), functions.col(TEMPORAL_JOIN_COLUMN)).otherwise(
                        feature_df[self.spine_time_key]
                    ),
                )
                .drop(self.spine_time_key)
                .withColumnRenamed(TEMPORARY_COLUMN_NAME, self.spine_time_key)
            )
        # Stable-sort columns in order of (join_keys, timestamp, feature_columns)
        # Note: Some columns may contain dot (.) in name. We can't select them without wrapping them in "`".
        return df.select(*[f"`{col}`" for col in sorted(df.columns, key=self._sort_column_key)])

    def _sort_column_key(self, col: str):
        """Join keys < time key < feature columns"""
        if col in self.fd.join_keys:
            return -1
        if col == self.spine_time_key:
            return 0
        return 1


def _get_time_limits_of_dataframe(df: DataFrame, time_key: str) -> Optional[pendulum.Period]:
    """The returned range is inclusive at the beginning & exclusive at the end: [start, end)."""
    # Fetch lower and upper time bound of the spine so that we can demand the individual feature definitions
    # to limit the amount of data they fetch from the raw data sources.
    # Returns None if df is empty.
    collected_df = df.select(
        functions.min(df[time_key]).alias("time_start"), functions.max(df[time_key]).alias("time_end")
    ).collect()
    time_start = collected_df[0]["time_start"]
    time_end = collected_df[0]["time_end"]
    if time_start is None or time_end is None:
        return None

    # Need to add 1 microsecond to the end time, since the range is exclusive at the end, and we need
    # to make sure to include the very last feature value (in terms of the event timestamp).
    return pendulum.instance(time_end).add(microseconds=1) - pendulum.instance(time_start)


def _sort_key_for_spine(fd_and_config):
    # Fully bound BFV/BWAFV will be processed first.
    # BFV/BWAFV with wildcard join_key will be processed next.
    # ODFVs will be processed last since they may have BFV/BWAFV inputs.
    fv = fd_and_config.feature_definition
    if fv.is_on_demand:
        return 3
    elif fv.wildcard_join_key is None:
        return 1
    return 2


def _sort_key_for_features(fs_config: "tecton_spark.feature_set_config.FeatureSetConfig"):  # type: ignore
    features = fs_config.features

    def sort_key(column):
        # all non-features will have -1 sort key, meaning they'll be stably ordered at the beginning of the dataframe
        # all features will come after the rest of the columns in the order of `features` coming from the FSC definition
        return features.index(column) if column in features else -1

    return sort_key


def get_features_for_spine(
    spark,
    spine_df: DataFrame,
    feature_set_config: FeatureSetConfig,
    timestamp_key: Optional[str],
    from_source: bool = False,
    include_feature_view_timestamp_columns: bool = False,
) -> DataFrame:
    """Get features dataframe given the spine and feature set.

    :param spark: Spark session instance.
    :param spine_df: Spine Dataframe.
    :param feature_set_config: FeatureSetConfig instance.
    :param timestamp_key: Name of the time column in spine_df.
    :param from_source: (Optional) Whether feature values should be recomputed from the original data source.
            If False, we will read the values from the materialized store.
    :param include_feature_view_timestamp_columns: (Optional) Include timestamp columns for every individual feature definition.
    :return: A Spark Dataframe containing feature values for each row in spine_df.
    """
    wildcard_key_not_in_spine = False
    all_spine_join_keys = set()
    # Do some early validation.
    spine_columns = [c.name for c in spine_df.schema]
    for fd_and_config in feature_set_config._get_feature_definitions_and_join_configs():
        if from_source and is_bfc_mode_single(fd_and_config.feature_definition):
            raise errors.FV_BFC_SINGLE_FROM_SOURCE

        # Validate that join key overrides are actually present in the spine SQL's columns.
        # This also ensures the union of join keys across FeatureViews is present in the spine.
        for spine_key, fv_key in fd_and_config.join_keys:
            if spine_key not in spine_columns:
                if fv_key == fd_and_config.feature_definition.wildcard_join_key:
                    wildcard_key_not_in_spine = True
                    wildcard_join_key = fd_and_config.feature_definition.wildcard_join_key
                else:
                    raise errors.FS_SPINE_JOIN_KEY_OVERRIDE_INVALID(spine_key, fv_key, spine_columns)
            else:
                all_spine_join_keys.add(spine_key)

    df = spine_df

    if timestamp_key is not None and timestamp_key not in spine_columns:
        raise errors.FS_SPINE_TIMESTAMP_KEY_INVALID(timestamp_key, spine_columns)

    spine_time_limits = _get_time_limits_of_dataframe(spine_df, timestamp_key) if timestamp_key is not None else None

    total = len(feature_set_config._get_feature_definitions_and_join_configs())
    num = 0
    for fd_and_config in sorted(
        feature_set_config._get_feature_definitions_and_join_configs(), key=_sort_key_for_spine
    ):
        definition = fd_and_config.feature_definition
        parent_df = df
        num += 1
        logger.info(f"{definition.name}: (FV {num}/{total}) Start Build [1/3]")
        df = _FeatureSetQueryBuilder(
            spark=spark,
            spine_df=spine_df,
            all_spine_join_keys=list(all_spine_join_keys),
            wildcard_key_not_in_spine=wildcard_key_not_in_spine,
            parent_df=parent_df,
            fd=definition,
            override_join_keys=dict(fd_and_config.join_keys),
            namespace=fd_and_config.namespace,
            features=fd_and_config.features,
            spine_time_key=timestamp_key,
            use_materialized_data=not from_source,
            include_feature_view_timestamp_columns=include_feature_view_timestamp_columns,
            join_type="left",
            spine_time_limits=spine_time_limits,
        ).build()

    # Seems reasonable that wildcard queries can return 0-k rows per original row in spine.
    if wildcard_key_not_in_spine:
        df = df.filter(functions.col(wildcard_join_key).isNotNull())

    df = df.select([f"`{column}`" for column in sorted(df.columns, key=_sort_key_for_features(feature_set_config))])
    return df


def _sort_key(fd_and_config):
    # The sort order is as follows:
    # 1. Temporal Aggregate features come before Temporal features.
    # 2. Among temporal aggregate features, shorter tile intervals come before longer tile intervals.
    # 3. No order between multiple temporal features.
    # This is an important invariant for _FeatureSetQueryBuilder._join to work (see comments there).
    fd = fd_and_config.feature_definition
    if fd.is_temporal_aggregate:
        return fd.get_tile_interval
    else:
        return pendulum.Duration.max


def _filter_df_for_preview(feature_set_config, df):
    all_join_keys = list(
        {
            spine_join_key
            for fd_and_config in feature_set_config._get_feature_definitions_and_join_configs()
            for spine_join_key, _ in fd_and_config.join_keys
        }
    )
    # Drop duplicates from preview, since it looks pretty bad if we
    # show same join keys with different timestamps in preview
    df = df.drop_duplicates(all_join_keys)
    return df


def _get_feature_dataframe_with_limits(
    fd: FeatureDefinition,
    spine: Union[DataFrame, pandas.DataFrame, None],
    spine_time_key: Optional[str],
    spine_time_limits: Optional[pendulum.Period],
    use_materialized_data: bool,
    wildcard_key_not_in_spine: bool = False,
    validate_time_key=True,
    namespace: Optional[str] = None,
) -> "tecton.interactive.data_frame.DataFrame":
    """
    Returns a Tecton DataFrame that contains the output Feature Transformation of the FeatureView.

    :param spine: (Optional) The spine to join against, either as SQL string or a dataframe.
        If present, the returned data frame will contain rollups for all (join key, temporal key)
        combinations that are required to compute a full frame from the spine.
    :param spine_time_key: (Optional) Name of the time column in spine.
        If unspecified, will default to the name of the timestamp key of the feature definition.
    :param spine_time_limits: (Optional) Spine Time Bounds, precomputed by the caller.
    :param wildcard_key_not_in_spine: Whether or not the wildcard join_key is present in the spine.
        Defaults to False if spine is not specified or if the feature definition has no wildcard join_key.
    :return: A Tecton DataFrame.
    """
    from tecton._internals.feature_views import aggregations
    from tecton._internals import utils
    import tecton

    tc = TectonContext.get_instance()

    spine_df = tc._get_spine_df(spine)

    if validate_time_key:
        if fd.is_on_demand:
            fv_time_key = None
            assert isinstance(spine_df, DataFrame), "Spine must be a DataFrame for OnDemandFeatureView"
        elif fd.is_temporal_aggregate:
            assert fd.trailing_time_window_aggregation
            fv_time_key = fd.trailing_time_window_aggregation.time_key
        else:
            fv_time_key = fd.timestamp_key

        # Validate using either the spine_time_key or fv_time_key
        if spine_time_key is not None:
            utils.validate_spine_dataframe(spine_df, spine_time_key)
            if fv_time_key is not None:
                spine_df = spine_df.withColumnRenamed(spine_time_key, fv_time_key)
        elif fv_time_key is not None:
            utils.validate_spine_dataframe(spine_df, fv_time_key)

    # spine_time_limits refers to the range of feature timestamps.
    # Converting to the corresponding raw data time range.
    raw_data_time_limits = None
    if fd.is_temporal_aggregate or fd.is_temporal:
        raw_data_time_limits = aggregations._get_time_limits(
            fd=fd, spine_df=spine_df, spine_time_limits=spine_time_limits
        )

    use_materialized_data = _validate_and_return_use_materialized_data(fd, use_materialized_data)

    try:
        if fd.is_temporal_aggregate:
            spark_df = aggregations.construct_full_tafv_df(
                tc._spark,
                time_aggregation=fd.trailing_time_window_aggregation,
                join_keys=fd.join_keys,
                feature_store_format_version=fd.get_feature_store_format_version,
                tile_interval=fd.get_tile_interval,
                fd=fd,
                spine_join_keys=fd.join_keys,
                wildcard_join_keys=fd.wildcard_join_key,
                spine_df=spine_df,
                use_materialized_data=use_materialized_data,
                raw_data_time_limits=raw_data_time_limits,
                wildcard_key_not_in_spine=wildcard_key_not_in_spine,
            )
        elif fd.is_on_demand:
            from tecton.interactive.feature_view import OnDemandFeatureView

            spark_df = OnDemandFeatureView(fd.fv, fd.fco_container)._on_demand_transform_dataframe(
                spine=spine_df,
                use_materialized_data=use_materialized_data,
                namespace=namespace,
            )
        elif fd.is_temporal:
            spark_df = aggregations.construct_full_tfv_or_ft_df(
                tc._spark,
                fd,
                spine_df=spine_df,
                use_materialized_data=use_materialized_data,
                raw_data_time_limits=raw_data_time_limits,
                wildcard_key_not_in_spine=wildcard_key_not_in_spine,
            )
        elif fd.is_feature_table:
            spark_df = aggregations.construct_full_tfv_or_ft_df(
                tc._spark,
                fd,
                spine_df=spine_df,
                use_materialized_data=True,
                raw_data_time_limits=raw_data_time_limits,
                wildcard_key_not_in_spine=wildcard_key_not_in_spine,
            )
        else:
            raise ValueError(f"Invalid feature type for FeatureView {fd.name}")
        return tecton.interactive.data_frame.DataFrame._create(spark_df)
    except AnalysisException as e:
        if "Unable to infer schema for Parquet" in e.desc or "doesn't exist" in e.desc:
            if fd.is_feature_table:
                return tecton.interactive.data_frame.DataFrame._create(
                    tc._spark.createDataFrame([], schema_to_spark(fd.view_schema))
                )
            else:
                raise errors.FV_NO_MATERIALIZED_DATA(fd.name)
        raise


def get_features_for_preview(
    spark,
    feature_set_config: "tecton_spark.feature_set_config.FeatureSetConfig",
    spine_time_key: Optional[str] = None,
    use_materialized_data: bool = True,
    include_feature_view_timestamp_columns: bool = False,
) -> DataFrame:
    """Get a synthetic features dataframe of all features for the given feature_set used for preview.

    :param spark: Spark session instance
    :param feature_set_config: FeatureSetConfig instance
    :param spine_time_key: (Optional) Name of the time column in the returning Dataframe.
        We default to one of the underlying feature definition's time column name.
    :param use_materialized_data: (Optional) Use materialized data if materialization is enabled
    :param include_feature_view_timestamp_columns: (Optional) Include timestamp columns for every individual feature definition.
    :return: A Spark Dataframe containing synthetic feature values from the given feature_set_config.
    """
    df = None
    for fd_and_config in sorted(feature_set_config._get_feature_definitions_and_join_configs(), key=_sort_key):
        # Use the first WAFV's time key as the time column name in the resulting DF
        if spine_time_key is None:
            spine_time_key = fd_and_config.feature_definition.timestamp_key
        df = _FeatureSetQueryBuilder(
            spark=spark,
            spine_df=None,
            parent_df=df,
            fd=fd_and_config.feature_definition,
            override_join_keys=dict(fd_and_config.join_keys),
            namespace=fd_and_config.namespace,
            features=fd_and_config.features,
            spine_time_key=spine_time_key,
            use_materialized_data=use_materialized_data,
            include_feature_view_timestamp_columns=include_feature_view_timestamp_columns,
            join_type="outer",
        ).build()
    return _filter_df_for_preview(feature_set_config, df)


def _validate_and_return_use_materialized_data(fd, use_materialized_data):
    if fd.is_on_demand:
        return False

    if use_materialized_data and not (fd.writes_to_offline_store and fd.materialization_enabled):
        raise errors.FD_GET_FEATURES_MATERIALIZATION_DISABLED(fd.name)

    return use_materialized_data



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/pair.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/common/pair.proto',
  package='tecton_proto.common',
  syntax='proto2',
  serialized_options=b'\n\021com.tecton.commonP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1etecton_proto/common/pair.proto\x12\x13tecton_proto.common\"4\n\x04Pair\x12\x14\n\x05\x66irst\x18\x01 \x01(\tR\x05\x66irst\x12\x16\n\x06second\x18\x02 \x01(\tR\x06secondB\x15\n\x11\x63om.tecton.commonP\x01'
)




_PAIR = _descriptor.Descriptor(
  name='Pair',
  full_name='tecton_proto.common.Pair',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='first', full_name='tecton_proto.common.Pair.first', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='first', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='second', full_name='tecton_proto.common.Pair.second', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='second', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=55,
  serialized_end=107,
)

DESCRIPTOR.message_types_by_name['Pair'] = _PAIR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Pair = _reflection.GeneratedProtocolMessageType('Pair', (_message.Message,), {
  'DESCRIPTOR' : _PAIR,
  '__module__' : 'tecton_proto.common.pair_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.common.Pair)
  })
_sym_db.RegisterMessage(Pair)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



"""
All of these objects are deprecated in the new API.
This file will get cleaned up when we stop supporting FWV1 SDK versions on the server side.
"""
from tecton.tecton_errors import TectonValidationError


class DataSourceConfig:
    """
    DEPRECATED, please use Tecton's V2 API: https://docs.tecton.ai/v2.
    """

    def __init__(self, *args, **kwargs) -> None:
        raise TectonValidationError("'DataSourceConfig' class is deprecated.")


class VirtualDataSource:
    """
    DEPRECATED: please use 'BatchDataSource' or 'StreamDataSource' from Tecton's V2 API: https://docs.tecton.ai/v2.
    """

    def __init__(self, *args, **kwargs) -> None:
        raise TectonValidationError(
            "'VirtualDataSource' class is deprecated. Please use 'BatchDataSource' or 'StreamDataSource' instead."
        )


def sql_transformation(*args, **kwargs) -> None:
    """
    DEPRECATED: please use 'transformation' from Tecton's V2 API: https://docs.tecton.ai/v2.
    """
    raise TectonValidationError("'sql_transformation' class is deprecated. Please use 'transformation' instead.")


def pyspark_transformation(*args, **kwargs) -> None:
    """
    DEPRECATED: please use 'transformation' from Tecton's V2 API: https://docs.tecton.ai/v2.
    """
    raise TectonValidationError("'pyspark_transformation' class is deprecated. Please use 'transformation' instead.")


def online_transformation(*args, **kwargs) -> None:
    """
    DEPRECATED: please use 'transformation' from Tecton's V2 API: https://docs.tecton.ai/v2.
    """
    raise TectonValidationError("'online_transformation' class is deprecated. Please use 'transformation' instead.")


class NewEMRClusterConfig:
    """
    DEPRECATED: please use 'EMRClusterConfig' from Tecton's V2 API: https://docs.tecton.ai/v2.
    """

    def __init__(self, *args, **kwargs) -> None:
        raise TectonValidationError("'NewEMRClusterConfig' class is deprecated. Please use 'EMRClusterConfig' instead.")


class NewDatabricksClusterConfig:
    """
    DEPRECATED: please use 'DatabricksClusterConfig' from Tecton's V2 API: https://docs.tecton.ai/v2.
    """

    def __init__(self, *args, **kwargs) -> None:
        raise TectonValidationError(
            "'NewDatabricksClusterConfig' class is deprecated. Please use 'DatabricksClusterConfig' instead."
        )


class MaterializationConfig:
    """
    DEPRECATED: please use Tecton's V2 API: https://docs.tecton.ai/v2.
    """

    def __init__(self, *args, **kwargs) -> None:
        raise TectonValidationError("'MaterializationConfig' class is deprecated.")


class TemporalAggregateFeaturePackage:
    """
    DEPRECATED: please use 'BatchWindowAggregateFeatureView' or 'StreamWindowAggregateFeatureView' from Tecton's V2 API: https://docs.tecton.ai/v2.
    """

    def __init__(self, *args, **kwargs) -> None:
        raise TectonValidationError(
            "'TemporalAggregateFeaturePackage' class is deprecated. Please use 'BatchWindowAggregateFeatureView' or 'StreamWindowAggregateFeatureView' instead."
        )


class TemporalFeaturePackage:
    """
    DEPRECATED: please use 'BatchFeatureView' or 'StreamFeatureView' from Tecton's V2 API: https://docs.tecton.ai/v2.
    """

    def __init__(self, *args, **kwargs) -> None:
        raise TectonValidationError(
            "'TemporalFeaturePackage' class is deprecated. Please use 'BatchFeatureView' or 'StreamFeatureView' instead."
        )


class OnlineFeaturePackage:
    """
    DEPRECATED: please use 'OnDemandFeatureView' from Tecton's V2 API: https://docs.tecton.ai/v2.
    """

    def __init__(self, *args, **kwargs) -> None:
        raise TectonValidationError(
            "'OnlineFeaturePackage' class is deprecated. Please use 'OnDemandFeatureView' instead."
        )


class PushFeaturePackage:
    """
    DEPRECATED: please use 'FeatureTable' from Tecton's V2 API: https://docs.tecton.ai/v2.
    """

    def __init__(self, *args, **kwargs) -> None:
        raise TectonValidationError("'PushFeaturePackage' class is deprecated. Please use 'FeatureTable' instead.")



from datetime import datetime
from datetime import timezone
from typing import Any
from typing import Callable
from typing import List

from google.protobuf import timestamp_pb2
from jinja2 import BaseLoader
from jinja2 import Environment
from texttable import Texttable

from tecton_proto.data.summary_pb2 import FcoSummary
from tecton_proto.data.summary_pb2 import SummaryItem


_TABLE_HTML_TEMPLATE = """
<table>
<thead>
    <tr>
    {% for heading in headings %}
    <th style='text-align: left;'>{{heading}}</th>
    {% endfor %}
    </tr>
</thead>
<tbody>
    {% for item in items %}
    <tr>
        {% for value in item %}
        {% if value is string and '\n' in value %}
        <td style='text-align: left;'><pre style='padding: 5px; border:1px solid #efefef'>{{value}}</pre></td>
        {% else %}
        <td style='text-align: left;'>{{value}}</td>
        {% endif %}
        {% endfor %}
    </tr>
    {% endfor %}
</tbody>
</table>
"""


def _fco_metadata_formatter(key, value):
    if isinstance(value, timestamp_pb2.Timestamp):
        t = datetime.fromtimestamp(value.ToSeconds())
        return t.astimezone(timezone.utc).strftime("%Y-%m-%d %H:%M:%S %Z")
    return value


class Displayable(object):
    _table_template = Environment(loader=BaseLoader(), autoescape=True).from_string(_TABLE_HTML_TEMPLATE)

    def __init__(self, text_table, html):
        self._text_table = text_table
        self._html = html

    def __repr__(self):
        return self._text_table.draw()

    def __str__(self):
        return self.__repr__()

    def _repr_html_(self):
        return self._html

    @classmethod
    def from_items(cls, headings, items, max_width=None, deco=Texttable.HEADER):
        """Returns a displayable table built from an iterable of tuples."""
        for row_index, item in enumerate(items):
            if len(item) != len(headings):
                raise ValueError(
                    f"malformed dimensions: headings has length {len(headings)}, but row {row_index} has lenght {len(item)}"
                )

        text_table = Texttable()
        if max_width is not None:
            text_table.set_max_width(max_width)
        text_table.add_rows(items, header=False)
        text_table.header(headings)
        text_table.set_deco(deco)

        # build HTML version
        html = cls._table_template.render(headings=headings, items=items)

        return cls(text_table, html)

    @classmethod
    def from_fco_summary(
        cls,
        fco_summary: FcoSummary,
        additional_items=[],
        value_formatter: Callable[[str, Any], str] = _fco_metadata_formatter,
    ):

        fco_metadata_fields = [
            "name",
            "workspace",
            "description",
            "created_at",
            "owner",
            "last_modified_by",
            "family",
            "source_filename",
            "tags",
        ]
        table_items = [
            (_transform_field_name(field), _fco_metadata_formatter(field, getattr(fco_summary.fco_metadata, field)))
            for field in fco_metadata_fields
        ]

        for item in fco_summary.summary_items:
            if not item.HasField("display_name"):
                continue
            if item.HasField("value"):
                value = item.value
            elif len(item.multi_values) > 0:
                value = ", ".join(item.multi_values)
            elif len(item.nested_summary_items) > 0:
                value = str(cls._get_nested_table(item.nested_summary_items))
            else:
                value = ""
            table_items.append((item.display_name, value_formatter(item.key, value)))

        for key, value in additional_items:
            table_items.append((key, value_formatter(key, value)))

        text_table = Texttable()
        text_table.add_rows(table_items, header=False)
        text_table.set_deco(Texttable.BORDER | Texttable.VLINES | Texttable.HLINES)
        text_table.set_cols_valign(["m", "m"])

        # build HTML version
        html = cls._table_template.render(headings=["", ""], items=table_items)

        return cls(text_table, html)

    @classmethod
    def _get_nested_table(cls, nested_summary_items: List[SummaryItem]):
        displayed_items = []
        for nested_item in nested_summary_items:
            if not nested_item.HasField("display_name"):
                continue
            if len(nested_item.nested_summary_items) > 0:
                displayed_items.append(
                    (nested_item.display_name, str(cls._get_nested_table(nested_item.nested_summary_items)))
                )
            else:
                displayed_items.append((nested_item.display_name, nested_item.value))

        text_table = Texttable()
        text_table.add_rows(displayed_items, header=False)
        text_table.set_deco(0)
        return text_table.draw()


def _transform_field_name(name):
    return name.replace("_", " ").title()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/feature_server/configuration/feature_server_configuration.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.args import user_defined_function_pb2 as tecton__proto_dot_args_dot_user__defined__function__pb2
from tecton_proto.args import pipeline_pb2 as tecton__proto_dot_args_dot_pipeline__pb2
from tecton_proto.common import aggregation_function_pb2 as tecton__proto_dot_common_dot_aggregation__function__pb2
from tecton_proto.common import column_type_pb2 as tecton__proto_dot_common_dot_column__type__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.data import feature_service_pb2 as tecton__proto_dot_data_dot_feature__service__pb2
from tecton_proto.data import feature_view_pb2 as tecton__proto_dot_data_dot_feature__view__pb2
from tecton_proto.data import tecton_api_key_pb2 as tecton__proto_dot_data_dot_tecton__api__key__pb2
from tecton_proto.data import new_transformation_pb2 as tecton__proto_dot_data_dot_new__transformation__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/feature_server/configuration/feature_server_configuration.proto',
  package='tecton_proto.feature_server.configuration',
  syntax='proto2',
  serialized_options=b'\n(com.tecton.feature_service.configurationP\001Z>github.com/tecton-ai/tecton_proto/feature_server/configuration',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\nLtecton_proto/feature_server/configuration/feature_server_configuration.proto\x12)tecton_proto.feature_server.configuration\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a-tecton_proto/args/user_defined_function.proto\x1a tecton_proto/args/pipeline.proto\x1a.tecton_proto/common/aggregation_function.proto\x1a%tecton_proto/common/column_type.proto\x1a\x1ctecton_proto/common/id.proto\x1a\'tecton_proto/data/feature_service.proto\x1a$tecton_proto/data/feature_view.proto\x1a&tecton_proto/data/tecton_api_key.proto\x1a*tecton_proto/data/new_transformation.proto\"\xeb\x01\n\x0c\x46\x65\x61turesPlan\x12[\n\x0c\x66\x65\x61ture_plan\x18\x01 \x01(\x0b\x32\x36.tecton_proto.feature_server.configuration.FeaturePlanH\x00R\x0b\x66\x65\x61turePlan\x12x\n\x17on_demand_features_plan\x18\x02 \x01(\x0b\x32?.tecton_proto.feature_server.configuration.OnDemandFeaturesPlanH\x00R\x14onDemandFeaturesPlanB\x04\n\x02\x66p\"\xdf\x01\n\x06\x43olumn\x12\x33\n\x04type\x18\x01 \x01(\x0e\x32\x1f.tecton_proto.common.ColumnTypeR\x04type\x12\x35\n\x17\x66\x65\x61ture_view_space_name\x18\x02 \x01(\tR\x14\x66\x65\x61tureViewSpaceName\x12;\n\x1a\x66\x65\x61ture_service_space_name\x18\x03 \x01(\tR\x17\x66\x65\x61tureServiceSpaceName\x12,\n\x12\x66\x65\x61ture_view_index\x18\x04 \x01(\x05R\x10\x66\x65\x61tureViewIndex\"\xe9\n\n\x0b\x46\x65\x61turePlan\x12V\n\routput_column\x18\x04 \x01(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x0coutputColumn\x12V\n\rinput_columns\x18\x01 \x03(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x0cinputColumns\x12[\n\x14\x61ggregation_function\x18\x03 \x01(\x0e\x32(.tecton_proto.common.AggregationFunctionR\x13\x61ggregationFunction\x12n\n\x1b\x61ggregation_function_params\x18\x06 \x01(\x0b\x32..tecton_proto.common.AggregationFunctionParamsR\x19\x61ggregationFunctionParams\x12H\n\x12\x61ggregation_window\x18\x05 \x01(\x0b\x32\x19.google.protobuf.DurationR\x11\x61ggregationWindow\x12N\n\tjoin_keys\x18\x07 \x03(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x08joinKeys\x12_\n\x12wildcard_join_keys\x18\x08 \x03(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x10wildcardJoinKeys\x12\x1d\n\ntable_name\x18\t \x01(\tR\ttableName\x12|\n\x19\x64\x61ta_table_timestamp_type\x18\x0f \x01(\x0e\x32\x41.tecton_proto.feature_server.configuration.DataTableTimestampTypeR\x16\x64\x61taTableTimestampType\x12\x82\x01\n\x1bstatus_table_timestamp_type\x18\x10 \x01(\x0e\x32\x43.tecton_proto.feature_server.configuration.StatusTableTimestampTypeR\x18statusTableTimestampType\x12#\n\rtimestamp_key\x18\x0c \x01(\tR\x0ctimestampKey\x12<\n\x0cslide_period\x18\r \x01(\x0b\x32\x19.google.protobuf.DurationR\x0bslidePeriod\x12:\n\x0bserving_ttl\x18\x0e \x01(\x0b\x32\x19.google.protobuf.DurationR\nservingTtl\x12\x30\n\x14refresh_status_table\x18\x11 \x01(\x08R\x12refreshStatusTable\x12*\n\x11\x66\x65\x61ture_view_name\x18\x15 \x01(\tR\x0f\x66\x65\x61tureViewName\x12?\n\x1c\x66\x65\x61ture_store_format_version\x18\x12 \x01(\x05R\x19\x66\x65\x61tureStoreFormatVersion\x12T\n\x13online_store_params\x18\x13 \x01(\x0b\x32$.tecton_proto.data.OnlineStoreParamsR\x11onlineStoreParams\x12\x1a\n\x08\x64\x65ltaTTL\x18\x14 \x01(\x03R\x08\x64\x65ltaTTLJ\x04\x08\x02\x10\x03J\x04\x08\n\x10\x0bJ\x04\x08\x0b\x10\x0c\"g\n\x11\x46\x65\x61tureVectorPlan\x12R\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32\x36.tecton_proto.feature_server.configuration.FeaturePlanR\x08\x66\x65\x61tures\"\x9c\x05\n\x14OnDemandFeaturesPlan\x12l\n\x19\x61rgs_from_request_context\x18\x02 \x03(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x16\x61rgsFromRequestContext\x12K\n\x07outputs\x18\x03 \x03(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x07outputs\x12\x83\x01\n\x12\x66\x65\x61ture_set_inputs\x18\x05 \x03(\x0b\x32U.tecton_proto.feature_server.configuration.OnDemandFeaturesPlan.FeatureSetInputsEntryR\x10\x66\x65\x61tureSetInputs\x12\x37\n\x08pipeline\x18\x06 \x01(\x0b\x32\x1b.tecton_proto.args.PipelineR\x08pipeline\x12N\n\x0ftransformations\x18\x07 \x03(\x0b\x32$.tecton_proto.data.NewTransformationR\x0ftransformations\x12*\n\x11\x66\x65\x61ture_view_name\x18\x08 \x01(\tR\x0f\x66\x65\x61tureViewName\x1a\x81\x01\n\x15\x46\x65\x61tureSetInputsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12R\n\x05value\x18\x02 \x01(\x0b\x32<.tecton_proto.feature_server.configuration.FeatureVectorPlanR\x05value:\x02\x38\x01J\x04\x08\x01\x10\x02J\x04\x08\x04\x10\x05\"\x81\x01\n\rLoggingConfig\x12\x1f\n\x0bsample_rate\x18\x01 \x01(\x02R\nsampleRate\x12.\n\x13log_effective_times\x18\x02 \x01(\x08R\x11logEffectiveTimes\x12\x1f\n\x0b\x61vro_schema\x18\x03 \x01(\tR\navroSchema\"\x8d\x05\n\x12\x46\x65\x61tureServicePlan\x12\x45\n\x12\x66\x65\x61ture_service_id\x18\x04 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10\x66\x65\x61tureServiceId\x12?\n\x0f\x66\x65\x61ture_view_id\x18\n \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12\x30\n\x14\x66\x65\x61ture_service_name\x18\x02 \x01(\tR\x12\x66\x65\x61tureServiceName\x12*\n\x11\x66\x65\x61ture_view_name\x18\t \x01(\tR\x0f\x66\x65\x61tureViewName\x12%\n\x0eworkspace_name\x18\x08 \x01(\tR\rworkspaceName\x12^\n\x0e\x66\x65\x61tures_plans\x18\r \x03(\x0b\x32\x37.tecton_proto.feature_server.configuration.FeaturesPlanR\rfeaturesPlans\x12N\n\x11join_key_template\x18\x05 \x01(\x0b\x32\".tecton_proto.data.JoinKeyTemplateR\x0fjoinKeyTemplate\x12\x41\n\x10\x66\x65\x61ture_view_ids\x18\x06 \x03(\x0b\x32\x17.tecton_proto.common.IdR\x0e\x66\x65\x61tureViewIds\x12_\n\x0elogging_config\x18\x0c \x01(\x0b\x32\x38.tecton_proto.feature_server.configuration.LoggingConfigR\rloggingConfigJ\x04\x08\x01\x10\x02J\x04\x08\x03\x10\x04J\x04\x08\x07\x10\x08J\x04\x08\x0b\x10\x0c\"\x80\x04\n\x11GlobalTableConfig\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x04 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12<\n\x0cslide_period\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0bslidePeriod\x12\x12\n\x04size\x18\x03 \x01(\x05R\x04size\x12\x82\x01\n\x1bstatus_table_timestamp_type\x18\x06 \x01(\x0e\x32\x43.tecton_proto.feature_server.configuration.StatusTableTimestampTypeR\x18statusTableTimestampType\x12\x30\n\x14refresh_status_table\x18\x07 \x01(\x08R\x12refreshStatusTable\x12?\n\x1c\x66\x65\x61ture_store_format_version\x18\x08 \x01(\x05R\x19\x66\x65\x61tureStoreFormatVersion\x12T\n\x13online_store_params\x18\t \x01(\x0b\x32$.tecton_proto.data.OnlineStoreParamsR\x11onlineStoreParamsJ\x04\x08\x01\x10\x02J\x04\x08\x05\x10\x06\"\xc6\x04\n\x1a\x46\x65\x61tureServerConfiguration\x12?\n\rcomputed_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0c\x63omputedTime\x12h\n\x10\x66\x65\x61ture_services\x18\x01 \x03(\x0b\x32=.tecton_proto.feature_server.configuration.FeatureServicePlanR\x0f\x66\x65\x61tureServices\x12\xa0\x01\n\x1bglobal_table_config_by_name\x18\x03 \x03(\x0b\x32\x62.tecton_proto.feature_server.configuration.FeatureServerConfiguration.GlobalTableConfigByNameEntryR\x17globalTableConfigByName\x12O\n\x13\x61uthorized_api_keys\x18\x04 \x03(\x0b\x32\x1f.tecton_proto.data.TectonApiKeyR\x11\x61uthorizedApiKeys\x1a\x88\x01\n\x1cGlobalTableConfigByNameEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12R\n\x05value\x18\x02 \x01(\x0b\x32<.tecton_proto.feature_server.configuration.GlobalTableConfigR\x05value:\x02\x38\x01*\x90\x01\n\x16\x44\x61taTableTimestampType\x12%\n!DATA_TABLE_TIMESTAMP_TYPE_UNKNOWN\x10\x00\x12&\n\"DATA_TABLE_TIMESTAMP_TYPE_SORT_KEY\x10\x01\x12\'\n#DATA_TABLE_TIMESTAMP_TYPE_ATTRIBUTE\x10\x02*\x98\x01\n\x18StatusTableTimestampType\x12\'\n#STATUS_TABLE_TIMESTAMP_TYPE_UNKNOWN\x10\x00\x12(\n$STATUS_TABLE_TIMESTAMP_TYPE_SORT_KEY\x10\x01\x12)\n%STATUS_TABLE_TIMESTAMP_TYPE_ATTRIBUTE\x10\x02\x42l\n(com.tecton.feature_service.configurationP\x01Z>github.com/tecton-ai/tecton_proto/feature_server/configuration'
  ,
  dependencies=[google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_user__defined__function__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_pipeline__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_aggregation__function__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_column__type__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_id__pb2.DESCRIPTOR,tecton__proto_dot_data_dot_feature__service__pb2.DESCRIPTOR,tecton__proto_dot_data_dot_feature__view__pb2.DESCRIPTOR,tecton__proto_dot_data_dot_tecton__api__key__pb2.DESCRIPTOR,tecton__proto_dot_data_dot_new__transformation__pb2.DESCRIPTOR,])

_DATATABLETIMESTAMPTYPE = _descriptor.EnumDescriptor(
  name='DataTableTimestampType',
  full_name='tecton_proto.feature_server.configuration.DataTableTimestampType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DATA_TABLE_TIMESTAMP_TYPE_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATA_TABLE_TIMESTAMP_TYPE_SORT_KEY', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATA_TABLE_TIMESTAMP_TYPE_ATTRIBUTE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=5066,
  serialized_end=5210,
)
_sym_db.RegisterEnumDescriptor(_DATATABLETIMESTAMPTYPE)

DataTableTimestampType = enum_type_wrapper.EnumTypeWrapper(_DATATABLETIMESTAMPTYPE)
_STATUSTABLETIMESTAMPTYPE = _descriptor.EnumDescriptor(
  name='StatusTableTimestampType',
  full_name='tecton_proto.feature_server.configuration.StatusTableTimestampType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STATUS_TABLE_TIMESTAMP_TYPE_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STATUS_TABLE_TIMESTAMP_TYPE_SORT_KEY', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STATUS_TABLE_TIMESTAMP_TYPE_ATTRIBUTE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=5213,
  serialized_end=5365,
)
_sym_db.RegisterEnumDescriptor(_STATUSTABLETIMESTAMPTYPE)

StatusTableTimestampType = enum_type_wrapper.EnumTypeWrapper(_STATUSTABLETIMESTAMPTYPE)
DATA_TABLE_TIMESTAMP_TYPE_UNKNOWN = 0
DATA_TABLE_TIMESTAMP_TYPE_SORT_KEY = 1
DATA_TABLE_TIMESTAMP_TYPE_ATTRIBUTE = 2
STATUS_TABLE_TIMESTAMP_TYPE_UNKNOWN = 0
STATUS_TABLE_TIMESTAMP_TYPE_SORT_KEY = 1
STATUS_TABLE_TIMESTAMP_TYPE_ATTRIBUTE = 2



_FEATURESPLAN = _descriptor.Descriptor(
  name='FeaturesPlan',
  full_name='tecton_proto.feature_server.configuration.FeaturesPlan',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_plan', full_name='tecton_proto.feature_server.configuration.FeaturesPlan.feature_plan', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featurePlan', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='on_demand_features_plan', full_name='tecton_proto.feature_server.configuration.FeaturesPlan.on_demand_features_plan', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='onDemandFeaturesPlan', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='fp', full_name='tecton_proto.feature_server.configuration.FeaturesPlan.fp',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=550,
  serialized_end=785,
)


_COLUMN = _descriptor.Descriptor(
  name='Column',
  full_name='tecton_proto.feature_server.configuration.Column',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='tecton_proto.feature_server.configuration.Column.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='type', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view_space_name', full_name='tecton_proto.feature_server.configuration.Column.feature_view_space_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewSpaceName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_service_space_name', full_name='tecton_proto.feature_server.configuration.Column.feature_service_space_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureServiceSpaceName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view_index', full_name='tecton_proto.feature_server.configuration.Column.feature_view_index', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewIndex', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=788,
  serialized_end=1011,
)


_FEATUREPLAN = _descriptor.Descriptor(
  name='FeaturePlan',
  full_name='tecton_proto.feature_server.configuration.FeaturePlan',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='output_column', full_name='tecton_proto.feature_server.configuration.FeaturePlan.output_column', index=0,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='outputColumn', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='input_columns', full_name='tecton_proto.feature_server.configuration.FeaturePlan.input_columns', index=1,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='inputColumns', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='aggregation_function', full_name='tecton_proto.feature_server.configuration.FeaturePlan.aggregation_function', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='aggregationFunction', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='aggregation_function_params', full_name='tecton_proto.feature_server.configuration.FeaturePlan.aggregation_function_params', index=3,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='aggregationFunctionParams', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='aggregation_window', full_name='tecton_proto.feature_server.configuration.FeaturePlan.aggregation_window', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='aggregationWindow', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='join_keys', full_name='tecton_proto.feature_server.configuration.FeaturePlan.join_keys', index=5,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='joinKeys', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='wildcard_join_keys', full_name='tecton_proto.feature_server.configuration.FeaturePlan.wildcard_join_keys', index=6,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='wildcardJoinKeys', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table_name', full_name='tecton_proto.feature_server.configuration.FeaturePlan.table_name', index=7,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='tableName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_table_timestamp_type', full_name='tecton_proto.feature_server.configuration.FeaturePlan.data_table_timestamp_type', index=8,
      number=15, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='dataTableTimestampType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status_table_timestamp_type', full_name='tecton_proto.feature_server.configuration.FeaturePlan.status_table_timestamp_type', index=9,
      number=16, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='statusTableTimestampType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp_key', full_name='tecton_proto.feature_server.configuration.FeaturePlan.timestamp_key', index=10,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampKey', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='slide_period', full_name='tecton_proto.feature_server.configuration.FeaturePlan.slide_period', index=11,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='slidePeriod', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='serving_ttl', full_name='tecton_proto.feature_server.configuration.FeaturePlan.serving_ttl', index=12,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='servingTtl', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='refresh_status_table', full_name='tecton_proto.feature_server.configuration.FeaturePlan.refresh_status_table', index=13,
      number=17, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='refreshStatusTable', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view_name', full_name='tecton_proto.feature_server.configuration.FeaturePlan.feature_view_name', index=14,
      number=21, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_store_format_version', full_name='tecton_proto.feature_server.configuration.FeaturePlan.feature_store_format_version', index=15,
      number=18, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureStoreFormatVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='online_store_params', full_name='tecton_proto.feature_server.configuration.FeaturePlan.online_store_params', index=16,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='onlineStoreParams', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deltaTTL', full_name='tecton_proto.feature_server.configuration.FeaturePlan.deltaTTL', index=17,
      number=20, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='deltaTTL', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1014,
  serialized_end=2399,
)


_FEATUREVECTORPLAN = _descriptor.Descriptor(
  name='FeatureVectorPlan',
  full_name='tecton_proto.feature_server.configuration.FeatureVectorPlan',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='features', full_name='tecton_proto.feature_server.configuration.FeatureVectorPlan.features', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='features', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2401,
  serialized_end=2504,
)


_ONDEMANDFEATURESPLAN_FEATURESETINPUTSENTRY = _descriptor.Descriptor(
  name='FeatureSetInputsEntry',
  full_name='tecton_proto.feature_server.configuration.OnDemandFeaturesPlan.FeatureSetInputsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='tecton_proto.feature_server.configuration.OnDemandFeaturesPlan.FeatureSetInputsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='key', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='tecton_proto.feature_server.configuration.OnDemandFeaturesPlan.FeatureSetInputsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3034,
  serialized_end=3163,
)

_ONDEMANDFEATURESPLAN = _descriptor.Descriptor(
  name='OnDemandFeaturesPlan',
  full_name='tecton_proto.feature_server.configuration.OnDemandFeaturesPlan',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='args_from_request_context', full_name='tecton_proto.feature_server.configuration.OnDemandFeaturesPlan.args_from_request_context', index=0,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='argsFromRequestContext', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='outputs', full_name='tecton_proto.feature_server.configuration.OnDemandFeaturesPlan.outputs', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='outputs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_set_inputs', full_name='tecton_proto.feature_server.configuration.OnDemandFeaturesPlan.feature_set_inputs', index=2,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureSetInputs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pipeline', full_name='tecton_proto.feature_server.configuration.OnDemandFeaturesPlan.pipeline', index=3,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='pipeline', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transformations', full_name='tecton_proto.feature_server.configuration.OnDemandFeaturesPlan.transformations', index=4,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='transformations', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view_name', full_name='tecton_proto.feature_server.configuration.OnDemandFeaturesPlan.feature_view_name', index=5,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_ONDEMANDFEATURESPLAN_FEATURESETINPUTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2507,
  serialized_end=3175,
)


_LOGGINGCONFIG = _descriptor.Descriptor(
  name='LoggingConfig',
  full_name='tecton_proto.feature_server.configuration.LoggingConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sample_rate', full_name='tecton_proto.feature_server.configuration.LoggingConfig.sample_rate', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sampleRate', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='log_effective_times', full_name='tecton_proto.feature_server.configuration.LoggingConfig.log_effective_times', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='logEffectiveTimes', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='avro_schema', full_name='tecton_proto.feature_server.configuration.LoggingConfig.avro_schema', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='avroSchema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3178,
  serialized_end=3307,
)


_FEATURESERVICEPLAN = _descriptor.Descriptor(
  name='FeatureServicePlan',
  full_name='tecton_proto.feature_server.configuration.FeatureServicePlan',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_service_id', full_name='tecton_proto.feature_server.configuration.FeatureServicePlan.feature_service_id', index=0,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureServiceId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view_id', full_name='tecton_proto.feature_server.configuration.FeatureServicePlan.feature_view_id', index=1,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_service_name', full_name='tecton_proto.feature_server.configuration.FeatureServicePlan.feature_service_name', index=2,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureServiceName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view_name', full_name='tecton_proto.feature_server.configuration.FeatureServicePlan.feature_view_name', index=3,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workspace_name', full_name='tecton_proto.feature_server.configuration.FeatureServicePlan.workspace_name', index=4,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workspaceName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='features_plans', full_name='tecton_proto.feature_server.configuration.FeatureServicePlan.features_plans', index=5,
      number=13, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featuresPlans', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='join_key_template', full_name='tecton_proto.feature_server.configuration.FeatureServicePlan.join_key_template', index=6,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='joinKeyTemplate', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view_ids', full_name='tecton_proto.feature_server.configuration.FeatureServicePlan.feature_view_ids', index=7,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewIds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='logging_config', full_name='tecton_proto.feature_server.configuration.FeatureServicePlan.logging_config', index=8,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='loggingConfig', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3310,
  serialized_end=3963,
)


_GLOBALTABLECONFIG = _descriptor.Descriptor(
  name='GlobalTableConfig',
  full_name='tecton_proto.feature_server.configuration.GlobalTableConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_view_id', full_name='tecton_proto.feature_server.configuration.GlobalTableConfig.feature_view_id', index=0,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='slide_period', full_name='tecton_proto.feature_server.configuration.GlobalTableConfig.slide_period', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='slidePeriod', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='tecton_proto.feature_server.configuration.GlobalTableConfig.size', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='size', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status_table_timestamp_type', full_name='tecton_proto.feature_server.configuration.GlobalTableConfig.status_table_timestamp_type', index=3,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='statusTableTimestampType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='refresh_status_table', full_name='tecton_proto.feature_server.configuration.GlobalTableConfig.refresh_status_table', index=4,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='refreshStatusTable', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_store_format_version', full_name='tecton_proto.feature_server.configuration.GlobalTableConfig.feature_store_format_version', index=5,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureStoreFormatVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='online_store_params', full_name='tecton_proto.feature_server.configuration.GlobalTableConfig.online_store_params', index=6,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='onlineStoreParams', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3966,
  serialized_end=4478,
)


_FEATURESERVERCONFIGURATION_GLOBALTABLECONFIGBYNAMEENTRY = _descriptor.Descriptor(
  name='GlobalTableConfigByNameEntry',
  full_name='tecton_proto.feature_server.configuration.FeatureServerConfiguration.GlobalTableConfigByNameEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='tecton_proto.feature_server.configuration.FeatureServerConfiguration.GlobalTableConfigByNameEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='key', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='tecton_proto.feature_server.configuration.FeatureServerConfiguration.GlobalTableConfigByNameEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4927,
  serialized_end=5063,
)

_FEATURESERVERCONFIGURATION = _descriptor.Descriptor(
  name='FeatureServerConfiguration',
  full_name='tecton_proto.feature_server.configuration.FeatureServerConfiguration',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='computed_time', full_name='tecton_proto.feature_server.configuration.FeatureServerConfiguration.computed_time', index=0,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='computedTime', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_services', full_name='tecton_proto.feature_server.configuration.FeatureServerConfiguration.feature_services', index=1,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureServices', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='global_table_config_by_name', full_name='tecton_proto.feature_server.configuration.FeatureServerConfiguration.global_table_config_by_name', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='globalTableConfigByName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='authorized_api_keys', full_name='tecton_proto.feature_server.configuration.FeatureServerConfiguration.authorized_api_keys', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='authorizedApiKeys', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_FEATURESERVERCONFIGURATION_GLOBALTABLECONFIGBYNAMEENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4481,
  serialized_end=5063,
)

_FEATURESPLAN.fields_by_name['feature_plan'].message_type = _FEATUREPLAN
_FEATURESPLAN.fields_by_name['on_demand_features_plan'].message_type = _ONDEMANDFEATURESPLAN
_FEATURESPLAN.oneofs_by_name['fp'].fields.append(
  _FEATURESPLAN.fields_by_name['feature_plan'])
_FEATURESPLAN.fields_by_name['feature_plan'].containing_oneof = _FEATURESPLAN.oneofs_by_name['fp']
_FEATURESPLAN.oneofs_by_name['fp'].fields.append(
  _FEATURESPLAN.fields_by_name['on_demand_features_plan'])
_FEATURESPLAN.fields_by_name['on_demand_features_plan'].containing_oneof = _FEATURESPLAN.oneofs_by_name['fp']
_COLUMN.fields_by_name['type'].enum_type = tecton__proto_dot_common_dot_column__type__pb2._COLUMNTYPE
_FEATUREPLAN.fields_by_name['output_column'].message_type = _COLUMN
_FEATUREPLAN.fields_by_name['input_columns'].message_type = _COLUMN
_FEATUREPLAN.fields_by_name['aggregation_function'].enum_type = tecton__proto_dot_common_dot_aggregation__function__pb2._AGGREGATIONFUNCTION
_FEATUREPLAN.fields_by_name['aggregation_function_params'].message_type = tecton__proto_dot_common_dot_aggregation__function__pb2._AGGREGATIONFUNCTIONPARAMS
_FEATUREPLAN.fields_by_name['aggregation_window'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_FEATUREPLAN.fields_by_name['join_keys'].message_type = _COLUMN
_FEATUREPLAN.fields_by_name['wildcard_join_keys'].message_type = _COLUMN
_FEATUREPLAN.fields_by_name['data_table_timestamp_type'].enum_type = _DATATABLETIMESTAMPTYPE
_FEATUREPLAN.fields_by_name['status_table_timestamp_type'].enum_type = _STATUSTABLETIMESTAMPTYPE
_FEATUREPLAN.fields_by_name['slide_period'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_FEATUREPLAN.fields_by_name['serving_ttl'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_FEATUREPLAN.fields_by_name['online_store_params'].message_type = tecton__proto_dot_data_dot_feature__view__pb2._ONLINESTOREPARAMS
_FEATUREVECTORPLAN.fields_by_name['features'].message_type = _FEATUREPLAN
_ONDEMANDFEATURESPLAN_FEATURESETINPUTSENTRY.fields_by_name['value'].message_type = _FEATUREVECTORPLAN
_ONDEMANDFEATURESPLAN_FEATURESETINPUTSENTRY.containing_type = _ONDEMANDFEATURESPLAN
_ONDEMANDFEATURESPLAN.fields_by_name['args_from_request_context'].message_type = _COLUMN
_ONDEMANDFEATURESPLAN.fields_by_name['outputs'].message_type = _COLUMN
_ONDEMANDFEATURESPLAN.fields_by_name['feature_set_inputs'].message_type = _ONDEMANDFEATURESPLAN_FEATURESETINPUTSENTRY
_ONDEMANDFEATURESPLAN.fields_by_name['pipeline'].message_type = tecton__proto_dot_args_dot_pipeline__pb2._PIPELINE
_ONDEMANDFEATURESPLAN.fields_by_name['transformations'].message_type = tecton__proto_dot_data_dot_new__transformation__pb2._NEWTRANSFORMATION
_FEATURESERVICEPLAN.fields_by_name['feature_service_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_FEATURESERVICEPLAN.fields_by_name['feature_view_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_FEATURESERVICEPLAN.fields_by_name['features_plans'].message_type = _FEATURESPLAN
_FEATURESERVICEPLAN.fields_by_name['join_key_template'].message_type = tecton__proto_dot_data_dot_feature__service__pb2._JOINKEYTEMPLATE
_FEATURESERVICEPLAN.fields_by_name['feature_view_ids'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_FEATURESERVICEPLAN.fields_by_name['logging_config'].message_type = _LOGGINGCONFIG
_GLOBALTABLECONFIG.fields_by_name['feature_view_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_GLOBALTABLECONFIG.fields_by_name['slide_period'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_GLOBALTABLECONFIG.fields_by_name['status_table_timestamp_type'].enum_type = _STATUSTABLETIMESTAMPTYPE
_GLOBALTABLECONFIG.fields_by_name['online_store_params'].message_type = tecton__proto_dot_data_dot_feature__view__pb2._ONLINESTOREPARAMS
_FEATURESERVERCONFIGURATION_GLOBALTABLECONFIGBYNAMEENTRY.fields_by_name['value'].message_type = _GLOBALTABLECONFIG
_FEATURESERVERCONFIGURATION_GLOBALTABLECONFIGBYNAMEENTRY.containing_type = _FEATURESERVERCONFIGURATION
_FEATURESERVERCONFIGURATION.fields_by_name['computed_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_FEATURESERVERCONFIGURATION.fields_by_name['feature_services'].message_type = _FEATURESERVICEPLAN
_FEATURESERVERCONFIGURATION.fields_by_name['global_table_config_by_name'].message_type = _FEATURESERVERCONFIGURATION_GLOBALTABLECONFIGBYNAMEENTRY
_FEATURESERVERCONFIGURATION.fields_by_name['authorized_api_keys'].message_type = tecton__proto_dot_data_dot_tecton__api__key__pb2._TECTONAPIKEY
DESCRIPTOR.message_types_by_name['FeaturesPlan'] = _FEATURESPLAN
DESCRIPTOR.message_types_by_name['Column'] = _COLUMN
DESCRIPTOR.message_types_by_name['FeaturePlan'] = _FEATUREPLAN
DESCRIPTOR.message_types_by_name['FeatureVectorPlan'] = _FEATUREVECTORPLAN
DESCRIPTOR.message_types_by_name['OnDemandFeaturesPlan'] = _ONDEMANDFEATURESPLAN
DESCRIPTOR.message_types_by_name['LoggingConfig'] = _LOGGINGCONFIG
DESCRIPTOR.message_types_by_name['FeatureServicePlan'] = _FEATURESERVICEPLAN
DESCRIPTOR.message_types_by_name['GlobalTableConfig'] = _GLOBALTABLECONFIG
DESCRIPTOR.message_types_by_name['FeatureServerConfiguration'] = _FEATURESERVERCONFIGURATION
DESCRIPTOR.enum_types_by_name['DataTableTimestampType'] = _DATATABLETIMESTAMPTYPE
DESCRIPTOR.enum_types_by_name['StatusTableTimestampType'] = _STATUSTABLETIMESTAMPTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FeaturesPlan = _reflection.GeneratedProtocolMessageType('FeaturesPlan', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESPLAN,
  '__module__' : 'tecton_proto.feature_server.configuration.feature_server_configuration_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.feature_server.configuration.FeaturesPlan)
  })
_sym_db.RegisterMessage(FeaturesPlan)

Column = _reflection.GeneratedProtocolMessageType('Column', (_message.Message,), {
  'DESCRIPTOR' : _COLUMN,
  '__module__' : 'tecton_proto.feature_server.configuration.feature_server_configuration_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.feature_server.configuration.Column)
  })
_sym_db.RegisterMessage(Column)

FeaturePlan = _reflection.GeneratedProtocolMessageType('FeaturePlan', (_message.Message,), {
  'DESCRIPTOR' : _FEATUREPLAN,
  '__module__' : 'tecton_proto.feature_server.configuration.feature_server_configuration_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.feature_server.configuration.FeaturePlan)
  })
_sym_db.RegisterMessage(FeaturePlan)

FeatureVectorPlan = _reflection.GeneratedProtocolMessageType('FeatureVectorPlan', (_message.Message,), {
  'DESCRIPTOR' : _FEATUREVECTORPLAN,
  '__module__' : 'tecton_proto.feature_server.configuration.feature_server_configuration_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.feature_server.configuration.FeatureVectorPlan)
  })
_sym_db.RegisterMessage(FeatureVectorPlan)

OnDemandFeaturesPlan = _reflection.GeneratedProtocolMessageType('OnDemandFeaturesPlan', (_message.Message,), {

  'FeatureSetInputsEntry' : _reflection.GeneratedProtocolMessageType('FeatureSetInputsEntry', (_message.Message,), {
    'DESCRIPTOR' : _ONDEMANDFEATURESPLAN_FEATURESETINPUTSENTRY,
    '__module__' : 'tecton_proto.feature_server.configuration.feature_server_configuration_pb2'
    # @@protoc_insertion_point(class_scope:tecton_proto.feature_server.configuration.OnDemandFeaturesPlan.FeatureSetInputsEntry)
    })
  ,
  'DESCRIPTOR' : _ONDEMANDFEATURESPLAN,
  '__module__' : 'tecton_proto.feature_server.configuration.feature_server_configuration_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.feature_server.configuration.OnDemandFeaturesPlan)
  })
_sym_db.RegisterMessage(OnDemandFeaturesPlan)
_sym_db.RegisterMessage(OnDemandFeaturesPlan.FeatureSetInputsEntry)

LoggingConfig = _reflection.GeneratedProtocolMessageType('LoggingConfig', (_message.Message,), {
  'DESCRIPTOR' : _LOGGINGCONFIG,
  '__module__' : 'tecton_proto.feature_server.configuration.feature_server_configuration_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.feature_server.configuration.LoggingConfig)
  })
_sym_db.RegisterMessage(LoggingConfig)

FeatureServicePlan = _reflection.GeneratedProtocolMessageType('FeatureServicePlan', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESERVICEPLAN,
  '__module__' : 'tecton_proto.feature_server.configuration.feature_server_configuration_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.feature_server.configuration.FeatureServicePlan)
  })
_sym_db.RegisterMessage(FeatureServicePlan)

GlobalTableConfig = _reflection.GeneratedProtocolMessageType('GlobalTableConfig', (_message.Message,), {
  'DESCRIPTOR' : _GLOBALTABLECONFIG,
  '__module__' : 'tecton_proto.feature_server.configuration.feature_server_configuration_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.feature_server.configuration.GlobalTableConfig)
  })
_sym_db.RegisterMessage(GlobalTableConfig)

FeatureServerConfiguration = _reflection.GeneratedProtocolMessageType('FeatureServerConfiguration', (_message.Message,), {

  'GlobalTableConfigByNameEntry' : _reflection.GeneratedProtocolMessageType('GlobalTableConfigByNameEntry', (_message.Message,), {
    'DESCRIPTOR' : _FEATURESERVERCONFIGURATION_GLOBALTABLECONFIGBYNAMEENTRY,
    '__module__' : 'tecton_proto.feature_server.configuration.feature_server_configuration_pb2'
    # @@protoc_insertion_point(class_scope:tecton_proto.feature_server.configuration.FeatureServerConfiguration.GlobalTableConfigByNameEntry)
    })
  ,
  'DESCRIPTOR' : _FEATURESERVERCONFIGURATION,
  '__module__' : 'tecton_proto.feature_server.configuration.feature_server_configuration_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.feature_server.configuration.FeatureServerConfiguration)
  })
_sym_db.RegisterMessage(FeatureServerConfiguration)
_sym_db.RegisterMessage(FeatureServerConfiguration.GlobalTableConfigByNameEntry)


DESCRIPTOR._options = None
_ONDEMANDFEATURESPLAN_FEATURESETINPUTSENTRY._options = None
_FEATURESERVERCONFIGURATION_GLOBALTABLECONFIGBYNAMEENTRY._options = None
# @@protoc_insertion_point(module_scope)



# nopycln: file
from tecton.interactive.data_source import BatchDataSource
from tecton.interactive.data_source import StreamDataSource
from tecton.interactive.dataset import Dataset
from tecton.interactive.entity import Entity
from tecton.interactive.feature_definition import FeatureDefinition
from tecton.interactive.feature_service import FeatureService
from tecton.interactive.feature_table import FeatureTable
from tecton.interactive.feature_view import BatchFeatureView
from tecton.interactive.feature_view import BatchWindowAggregateFeatureView
from tecton.interactive.feature_view import FeatureView
from tecton.interactive.feature_view import OnDemandFeatureView
from tecton.interactive.feature_view import StreamFeatureView
from tecton.interactive.feature_view import StreamWindowAggregateFeatureView
from tecton.interactive.transformation import Transformation
